@@ -3,6 +3,8 @@ import numpy as np
 import pandas as pd
 from sklearn.base import BaseEstimator
 from sklearn.linear_model import LogisticRegression
+import statsmodels.api as sma
+import statsmodels.formula.api as smf
 
 from .transform import WOETransformer, Combiner
 from .utils import to_ndarray, bin_by_splits, save_json, read_json
@@ -15,16 +17,17 @@ FACTOR_EMPTY = 'MISSING'
 FACTOR_UNKNOWN = 'UNKNOWN'
 
 
-
 class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
-    def __init__(self, pdo = 60, rate = 2, base_odds = 35, base_score = 750,
-        card = None, combiner = {}, transer = None, **kwargs):
+    def __init__(self, pdo = 40, rate = 2, base_odds = 1/60, base_score = 600, combiner = {}, transfer = None, model_type='lr', **kwargs):
         """
 
         Args:
             card (dict|str|IOBase): dict of card or io to read json
             combiner (toad.Combiner)
-            transer (toad.WOETransformer)
+            transfer (toad.WOETransformer)
+            # self.offset = 363.7244
+            # self.factor = 57.7078
+            model_type: lr or ols  lr:sklearn.LogisticRegression   ols:statsmodels.ols
         """
         self.pdo = pdo
         self.rate = rate
@@ -32,36 +35,35 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
         self.base_score = base_score
 
         self.factor = pdo / np.log(rate)
-        self.offset = base_score - self.factor * np.log(base_odds)
+        self.offset = base_score + self.factor * np.log(base_odds)
 
-        # self.combiner = combiner
         self._combiner = combiner
-        self.transer = transer
-        self.model = LogisticRegression(solver='lbfgs',**kwargs)
+        self.transfer = transfer
+        self.model_type = model_type
+
+        if self.model_type == 'ols':
+            self.model = sma
+        else:
+            self.model = LogisticRegression(**kwargs)
 
         self._feature_names = None
 
-        if card is not None:
-            # self.generate_card(card = card)
-            import warnings
-            warnings.warn(
-                """`ScoreCard(card = {.....})` will be deprecated soon,
-                    use `ScoreCard().load({.....})` instead!
-                """,
-                DeprecationWarning,
-            )
-
-            self.load(card)
         
     @property
     def coef_(self):
         """ coef of LR code
         """
-        return self.model.coef_[0]
+        if self.model_type == 'ols':
+            return self.model.params[1:]
+        else:
+            return self.model.coef_[0]
     
     @property
     def intercept_(self):
-        return self.model.intercept_[0]
+        if self.model_type == 'ols':
+            return self.model.params[0]
+        else:
+            return self.model.intercept_[0]
     
     @property
     def n_features_(self):
@@ -96,15 +98,20 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
         self._feature_names = X.columns.tolist()
 
         for f in self.features_:
-            if f not in self.transer:
-                raise Exception('column \'{f}\' is not in transer'.format(f = f))
+            if f not in self.transfer:
+                raise Exception('column \'{f}\' is not in transfer'.format(f = f))
 
-        self.model.fit(X, y)
+        if self.model_type == 'ols':
+            # 增加常数项,截距
+            X = sma.add_constant(X)
+            self.model = self.model.Logit(y, X).fit()
+            print(self.model.summary())
+        else:
+            self.model.fit(X, y)
 
         self.rules = self._generate_rules()
 
         return self
-    
 
     def predict(self, X, **kwargs):
         """predict score
@@ -116,23 +123,47 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
             array-like: predicted score
             DataFrame: sub score for each feature
         """
-        bins = self.combiner.transform(X[self.features_])
-        return self.bin_to_score(bins, **kwargs)
+
+        if self.model_type == 'ols':
+            X = sma.add_constant(X)
+            prob = self.model.predict(X)
+        else:
+            prob = self.model.predict_proba(X)[:, 1]
+
+        result = self.proba_to_score(prob)
+        result = np.around(result, decimals=2)
+        return result
+
+
+    def predict_prob(self, X, **kwargs):
+
+        if self.model_type == 'ols':
+            X = sma.add_constant(X)
+            prob = self.model.predict(X)
+        else:
+            prob = self.model.predict_proba(X)[:, 1]
+        return prob
     
 
     def _generate_rules(self):
-        if not self._check_rules(self.combiner, self.transer):
+        if not self._check_rules(self.combiner, self.transfer):
             raise Exception('generate failed')
         
         rules = {}
+        rules['intercept'] = {
+            'bins': np.ndarray([0]),
+            'woes': 0,
+            'weight': 0,
+            'scores': (self.offset - self.factor * self.intercept_,) * 1
+        }
 
         for idx, key in enumerate(self.features_):
             weight = self.coef_[idx]
 
             if weight == 0:
                 continue
-            
-            woe = self.transer[key]['woe']
+
+            woe = self.transfer[key]['woe']
             
             rules[key] = {
                 'bins': self.combiner[key],
@@ -140,20 +171,20 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
                 'weight': weight,
                 'scores': self.woe_to_score(woe, weight = weight),
             }
-        
+
         return rules
 
 
-    def _check_rules(self, combiner, transer):
+    def _check_rules(self, combiner, transfer):
         for col in self.features_:
             if col not in combiner:
                 raise Exception('column \'{col}\' is not in combiner'.format(col = col))
             
-            if col not in transer:
-                raise Exception('column \'{col}\' is not in transer'.format(col = col))
+            if col not in transfer:
+                raise Exception('column \'{col}\' is not in transfer'.format(col = col))
 
             l_c = len(combiner[col])
-            l_t = len(transer[col]['woe'])
+            l_t = len(transfer[col]['woe'])
 
             if l_c == 0:
                 continue
@@ -172,16 +203,17 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
         """covert probability to score
         
         odds = (1 - prob) / prob
-        score = factor * log(odds) * offset
+        score = offset - factor * log(odds)
         """
         return self.factor * (np.log(1 - prob) - np.log(prob)) + self.offset
 
-
     def bin_to_score(self, bins, return_sub = False):
         """predict score from bins
         """
         res = bins.copy()
         for col in self.rules:
+            if col == 'intercept':
+                continue
             s_map = self.rules[col]['scores']
             b = bins[col].values
             # set default group to min score
@@ -197,7 +229,6 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
         return score, res
 
 
-
     def woe_to_score(self, woe, weight = None):
         """calculate score by woe
         """
@@ -214,8 +245,8 @@ class ScoreCard(BaseEstimator, RulesMixin, BinsMixin):
         if isinstance(weight, np.ndarray):
             mask = (weight != 0).astype(int)
 
-        return (s + b / self.n_features_) * mask
-
+        # return (s + b / self.n_features_) * mask
+        return s * mask
 
     def _parse_rule(self, rule, **kwargs):
         bins = self.parse_bins(list(rule.keys()))
